; getc.asm
; This service routine retrieves a character from the keyboard controller; It
;         services the getc service call (interrupt #32) (The IN call is 
;         serviced by trap #35, promptgetc.)
; Read Algorithm: Spin on the keyboard controller ready bit (the high bit). 
;         When it becomes 1 (ready), there is a character in the data register
;         to grab.  When a read is performed on that location, the ready bit
;         drops back to zero until the next character is read.  (This assumes
;         that interrupt operation is not enabled. In this state, the hardware
;         does not guarantee that all characters are received.  In interrupt
;         operation, the OS provides a keyboard buffer.)
; Returns: Character from the keyboard in R0, upper half of R0 is 0.
; Context Information: R0 is changed, R1 and R2 are saved, R7 is lost in
;         the jump to this routine

                .ORIG   $0400           	; where this ISR resides
		ST	R7, SaveR7		; save R7 for later return

ReadStatusReg	LD	R0, KBSR	; get status addr
		LDR	R0, R0, #0      ; get status 
		BRZP	ReadStatusReg	; spin until ready

ReadDataReg	LD	R0, KBDR	; read the keystroke addr
		LDR	R0, R0, #0      ; read the keystroke

; return from interrupt call
Return		LD	R7, SaveR7
		RET			; return from trap

; Register locations
KBSR		.FILL	$FE00		; address of keyboard status register
KBDR		.FILL	$FE02		; address of keyboard data register
; Other data
SaveR7		.FILL	$0000
		.END






