; halt.asm
; This system call code halts the LC-2. It handles the HALT service call
;	(trap vector $25). It clears bit 15 of location $FFFF, the machine
;	control register (MCR), which is the clock enable bit for the 
;	processor (the run flip flop). When cleared, the clock signal is 
;	inhibited.

		.ORIG	$FD70		; where this routine resides
		ST      R7, SaveR7	; save R7 so we can RET at end of this
		ST	R1, SaveR1	; R1 used to change machine control reg
		ST	R0, SaveR0	; R0 is used as working space

; print message that machine is halting
		LEA	R0, Message
		PUTS

; clear bit 15 at $FFFF to stop the machine
		LD	R1, MCR		; load MC register contents into R1
		LDR	R1, R1, #0
		LD	R0, MASK	; R0 = $7FFF
		AND	R0, R1, R0	; mask to clear the top bit
		LD	R1, MCR		; store new value to MC register
		STR	R0, R1, #0

; The machine never reaches the following instructions because it is halted.
; We just put them in to make the routine look complete.

		LD	R0, SaveR0	; restore regs
		LD	R1, SaveR1
		LD	R7, SaveR7	; restore trap return
		RET

; Some constants
SaveR0		.FILL	$0000
SaveR1		.FILL	$0000
SaveR7		.FILL	$0000
Message		.FILL	"\n----- Halting the processor -----\n"
MCR		.FILL	$FFFE	; location of machine control register
MASK		.FILL	$7FFF	; mask to clear the top bit (run flip flop)
		.END

; reserve ~60 words for this routine.
