;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; promptgetc.asm -- LC-2 Character Input
;
; Trap routine to print a prompt and retrieve a character from the keyboard
;	  controller. It services the IN service call, which is trap vector
;         $23. The character retrieved is echoed to the screen.
;
; Inputs:  None
; Returns: Character from the keyboard in R0, upper half of R0 is 0.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                .ORIG   $04A0           ; system call starting address
		ST      R7, SaveR7	; save R7 so we can RET at end of this
		ST	R3, SaveR3	; R3 is used in the WriteChar subrtn
		ST	R2, SaveR2	; R2 will temporarily save the kbd char
		ST	R1, SaveR1	; R1 will point to prompt string
		; don't need to save R0--it is to be changed anyway

; write a newline, then the prompt
		LD	R0, NewLine
		JSR	WriteChar	; subroutine to write character in R0
		LEA	R1, Prompt	; get effective address of prompt str.
	LOOP	LDR	R0, R1, #0	; retrieve char pointed to by PromptPtr
		BRZ	ReadStatusReg	; zero word indicates end of string
		JSR	WriteChar
		ADD	R1, R1, #1	; increment pointer
		BR	LOOP		; write the next character

; read the character, echo it back to screen
ReadStatusReg	LD	R3, KBSR	; get status of keyboard
		LDR	R3, R3, #0
		BRZP	ReadStatusReg	; spin until ready

ReadDataReg	LD	R0, KBDR	; read keystroke into R0
		LDR	R0, R0, #0
		ADD	R2, R0, #0	; copy R0 into R2
		JSR	WriteChar	; echo character in R0
		LD	R0, NewLine	; write newline
		JSR	WriteChar

; return from trap
Return		ADD	R0, R2, #0	; copy character back from R2 to R0
		LD	R1, SaveR1	; restore registers
		LD	R2, SaveR2
		LD	R3, SaveR3
		LD      R7, SaveR7	; restore jump return R7
		RET			; return from trap

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Subroutine WriteChar writes the contents of R0 to the screen. R3 is 
;  overwritten. Though this is a very short subroutine, it is nonetheless 
;  instructive.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
WriteChar	ST	R7, SaveR7WriteChar
		LD	R3, DSR	; get status
		LDR	R3, R3, #0
		BRZP	WriteChar	; spin until ready

WriteIt		LD	R3, DDR		; write character
		STR	R0, R3, #0
		LD	R7, SaveR7WriteChar
		RET				; return to caller

SaveR7WriteChar	.FILL	$0000			; R7 save area for subroutine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; data and working space

DDR		.FILL		$FE06	; address of video (CRT) data register
DSR		.FILL		$FE04	; address of video status register
KBDR		.FILL		$FE02	; address of keyboard data register
KBSR		.FILL		$FE00	; address of keyboard status register
NewLine		.FILL		$000A	; ASCII newline character is $0A
SaveR1		.FILL		$0000
SaveR2		.FILL		$0000
SaveR3		.FILL		$0000
SaveR7		.FILL		$0000
Prompt		.STRINGZ	"Input a character>"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		.END
