; puts.asm
; This service routine writes a NULL-terminated string to the console.  It 
;         services the puts service call (trap $22). A NULL termination
;         is a 0 word after the last word in the string. No newline character
;	  is appended to the output (contrary to the standard C puts call).
; Inputs: R0 is a pointer to the string to print.
; Returns: none
; Context Information: R0, R1, and R2 are saved, and R7 is lost in the jump to
;         this routine

		.ORIG	$0450		; where this ISR resides
		ST	R7, SaveR7	; save R7 for later return
		ST	R0, SaveR0	; save other registers
		ST	R1, SaveR1
		ST	R2, SaveR2

; loop through each character in the array
DoAgain		LDR	R1, R0, #0	; retrieve the character(s)
		BRZ	Return		; if this word is 0, done
L2		LD	R2, DSR	; spin if video not ready
		LDR	R2, R2, #0
		BRZP	L2

		LD	R2, DDR	; write the character
		STR	R1, R2, #0
		ADD	R0, R0, #1	; increment pointer
		BR	DoAgain		; do it all over again

; return from interrupt call
Return		LD	R0, SaveR0	; restore registers
		LD	R1, SaveR1
		LD	R2, SaveR2
		LD	R7, SaveR7
		RET

; Register locations
DSR		.FILL	$FE04		; video status register
DDR		.FILL	$FE06		; to write characters to the screen
HPos		.FILL	$F3FD		; current cursor position (horizontal)
VPos		.FILL	$F3FE		; current cursor position (vertical)
; Working memory
SaveR0		.FILL	$0000		; space to save registers
SaveR1		.FILL	$0000
SaveR2		.FILL	$0000
SaveR7		.FILL	$0000
		.END

