; putspacked.asm
; This service routine writes a NULL-terminated string to the console.  It 
;         services the putspacked service call (trap #36). A NULL 
;	  termination MUST be either a 0 in the high byte of a word, after the
;	  last character in the string, or a complete 0 word. Printing ends 
;	  with a newline character.
; Inputs: R0 is a pointer to the string to print.
; Returns: none
; Context Information: R0, R1, and R2 are saved, and R7 is lost in the jump to
;         this routine

		.ORIG	$04E0		; where this ISR resides
		ST	R7, SaveR7	; save R7 for later return
		ST	R0, SaveR0	; save other registers
		ST	R1, SaveR1
		ST	R2, SaveR2
		ST	R3, SaveR3
		ADD	R1, R0, #0	; copy string pointer to R1

; loop through each character in the array
DoAgain		LDR	R0, R1, #0		; retrieve the character(s)
		BRZ	Return			; if this word is 0, done
	; otherwise, if the high byte is zero, we will be done after writing
	;  the next character
		JSR	WriteChar		; write character
	; hardware won't write the 0 in the MSB, if it is there
		LD	R2, ClearLowMask
		AND	R0, R0, R2		; mask off lower byte of word
		BRZ	Return			; if result is 0, done
		ADD	R1, R1, #1		; increment pointer
		BR	DoAgain			; do it all over again

; return from interrupt call
Return		LD	R0, NewLine	; write newline to screen
		JSR	WriteChar
		LD	R0, SaveR0	; restore R0 (puts is not reentrant)
		LD	R1, SaveR1
		LD	R2, SaveR2
		LD	R3, SaveR3
		LD	R7, SaveR7
		RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Subroutine WriteChar writes the contents of R0 to the screen. R3 is 
;  overwritten. Though this is very short, it is nonetheless instructive.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
WriteChar	ST	R7, SaveR7WriteChar
		LD	R3, DSR	; get status
		LDR	R3, R3, #0
		BRN	WriteIt		; bit 15 says video is ready
		BR	WriteChar	; otherwise, keep polling


WriteIt		LD	R3, DDR	; write newline character
		STR	R0, R3, #0
		LD	R7, SaveR7WriteChar
		RET			; return to caller
SaveR7WriteChar	.FILL	$0000		; R7 save area for subroutine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Register locations
DDR		.FILL	$FE06		; to write characters to the screen
DSR		.FILL	$FE04		; to check video status
HPos		.FILL	$F3FD		; current cursor position (horizontal)
VPos		.FILL	$F3FE		; current cursor position (vertical)
; Other data
ClearLowMask	.FILL	$FF00		; mask to clear low word
NewLine		.FILL	$000A		; hex for newline character
SaveR0		.FILL	$0000
SaveR1		.FILL	$0000
SaveR2		.FILL	$0000
SaveR3		.FILL	$0000
SaveR7		.FILL	$0000
		.END

; reserve ~48 words for this routine.
